
# q-expansion of newform 828.2.c.f, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 828
weight = 2
poly_data = [1516, 3288, -94, -5508, 668, 2192, -235, -382, 49, 20, 7, -6, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-4154576737, 1202967867, 5070158701, 1109629698, -1231457277, -321316858, 121854357, 38222699, -4946728, -5247742, 1105913, 88442], 1067332921], [[-1742855360, 343506438, -2130474782, 130617660, 1120725618, -31688011, -235686910, 32090090, 5949918, 6401729, -3840274, 589208], 2134665842], [[18089930960, 22485345216, -21566245414, -10192897058, 6611108320, 1709203913, -1013901050, -67206299, 14197330, 43816423, -15642992, 1551363], 2134665842], [[-7917342992, -8259228804, 9521675892, 2193358010, -3817048728, -14683221, 671406930, -70452542, -24462718, -18465355, 11372482, -1834094], 2134665842], [[-984295302, 396800192, -3849559026, 482742706, 2120971211, -226587922, -403192532, 57335395, 17414378, 8530376, -6619103, 1085283], 1067332921], [[-3027893328, 374726450, 9673073788, -3570962864, -4703029534, 1148248981, 863083944, -167280060, -55150274, -10461551, 14478294, -2759774], 2134665842], [[37559816, 1220234, -205970880, 63677032, 99492694, -18598401, -19610605, 3610340, 1011546, 311847, -321981, 58542], 40276714], [[2587702295, 79475086, -5913653120, 1696124762, 2959742934, -554227193, -562235748, 101092833, 29441909, 8935805, -9260117, 1674491], 1067332921], [[-79861492, 8395649106, 9340502812, -9434990032, -6101021978, 2604828985, 1145359229, -315974843, -92248346, -3027299, 19871151, -4311137], 2134665842], [[-12677532482, -20652949518, 24234677550, 5958731876, -9817971770, -573421752, 1881766562, -169458485, -46763110, -57015518, 30819072, -4525947], 2134665842], [[7646123486, 13689987652, 20918602, -12882383142, -3849486672, 3327229897, 945452125, -403063231, -86251826, 8244417, 17767085, -4525947], 2134665842]]

hecke_ring_character_values = [[415, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [461, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [649, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, -1, -1, 0, 0, -1, 0, 0, 0, 0, 0], [-1, 0, 0, 0, 0, 0, -1, 1, 1, 1, 0, 0], [2, 1, 1, 1, 1, 0, 1, 0, -1, -1, 0, 0], [-1, 1, 1, 0, -1, 1, 0, 1, 0, 0, 1, 1], [-1, 0, -1, 0, 0, 1, -2, 0, 1, 1, 0, 0], [0, 0, 2, 1, 0, 1, 0, -1, 0, -2, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [1, 0, 1, 2, 0, 2, 1, -1, -1, -2, 0, 0], [0, 0, 1, 2, 0, 1, 1, -2, 0, 0, 0, 0], [-3, 0, 0, -1, -2, 1, 1, 0, 1, 1, 0, 0], [1, 0, -1, 0, 0, 0, 1, -3, -1, 0, 0, 0], [0, 0, -2, -3, 0, -1, -2, 0, 0, 0, -1, 1], [0, 0, -1, 0, 0, -1, -1, 0, 0, 0, 0, 0], [0, 0, -1, 1, 0, 2, -1, -3, 0, 0, 1, -1], [-1, 2, 2, 0, -2, 1, 2, 0, -1, 0, 0, 0], [-5, 0, 0, 1, 2, 1, 1, 1, 1, -1, 1, 1], [0, 0, -2, -1, 0, 1, -2, 0, 0, 0, 1, -1], [-6, -2, -3, 0, 2, -3, -5, 0, 0, 0, 0, 0], [-1, -2, -3, -2, -2, 0, -1, 0, 3, 2, 0, 0], [1, 0, 2, 0, 0, -2, 3, 3, -1, -1, 0, 0], [0, 1, -1, 1, 1, -2, 1, -1, -1, -1, -1, -1], [-1, 0, -5, -2, 0, 1, -4, 2, 1, 3, 0, 0], [2, 1, -1, 0, -1, 0, 4, -1, 1, 0, -1, -1], [0, 0, -1, -2, 0, -3, 1, 7, 0, 2, 2, -2], [-1, 0, 4, 2, 0, 0, 1, 5, 1, -1, 0, 0], [-4, 1, -1, -1, -3, -2, -5, 1, -1, 1, 1, 1], [-1, 1, -3, 1, 1, -1, -1, 2, 2, -1, 2, 2], [-1, 0, 3, 2, 0, 1, 0, -2, 1, -1, 0, 0], [0, 0, 2, 4, 0, 4, 0, -2, 0, -2, 0, 0], [6, 0, -2, 2, 4, -4, 0, -2, -2, -2, -2, -2], [-1, 0, 3, 0, 0, -1, 0, 4, 1, -1, 2, -2], [1, 0, 0, 0, 0, -1, 2, -2, -1, 0, -2, 2], [2, 0, -1, -1, 0, -2, 3, 5, -2, 0, -1, 1], [2, 0, -1, 2, 0, 3, 1, 0, -2, -2, 0, 0], [9, 2, 2, -1, -4, 3, 3, 1, 1, 1, 1, 1], [1, 0, -2, 2, 0, 3, 0, -4, -1, 0, -2, 2], [4, 4, 3, 0, -4, 3, 3, 2, 0, 0, 2, 2], [-1, 0, 3, -2, 0, -2, -1, 7, 1, -2, -2, 2], [-1, 0, 6, 2, 4, 3, 2, 0, -3, -2, 0, 0], [-5, 1, 1, 1, 1, 1, 5, -1, 0, -1, -1, -1], [-7, -1, -1, 0, 1, 3, 4, 1, 4, 0, 1, 1], [-2, -1, 1, -2, -3, 2, -2, 1, 1, 2, 1, 1], [-2, 0, -1, 0, 0, 3, -5, -1, 2, 0, -2, 2], [1, 0, -4, -2, 0, -2, 1, 1, -1, 3, 0, 0], [-2, 0, 2, 2, 0, 2, -2, -2, 2, 0, 0, 0], [4, 0, 1, 6, 0, 3, 7, 0, -4, -2, 0, 0], [1, 0, -6, -1, -2, -1, 1, 1, 5, 1, 1, 1], [7, -3, -3, 1, 5, -3, -3, -1, 0, -1, -1, -1], [-2, 0, 4, 0, 0, 0, -2, -1, 2, -2, 2, -2], [9, -2, 2, 0, 2, -1, 2, -4, -3, 0, -4, -4], [0, -3, -1, -2, -1, 2, -2, 1, 3, 2, 1, 1], [9, 2, 2, -1, -4, 3, 1, 2, 1, 1, 2, 2], [0, 0, -1, 0, 0, 1, -1, 1, 0, 0, 2, -2], [3, 3, -1, 0, -3, 1, 0, 3, 2, 0, 3, 3], [0, 0, 5, 0, 0, -1, 1, 1, 0, -4, 0, 0], [-2, 0, 2, -4, 0, -2, -4, -2, 2, -2, 0, 0], [7, -2, 0, 2, 6, -1, 2, -2, -1, -2, -2, -2], [5, 0, -5, 0, 0, -1, 6, -6, -5, 1, 0, 0], [0, 0, -2, -1, 0, -1, 0, -5, 0, 2, 2, -2], [2, 0, 3, 5, 0, 0, 7, -7, -2, 0, 1, -1], [3, 0, 2, 2, 0, 1, 4, -10, -3, -4, 0, 0], [-2, 4, 2, 4, 4, 0, 2, 2, -2, -4, 2, 2], [-8, -2, -2, 0, 2, -2, -4, 0, 0, 0, 0, 0], [-1, 0, 5, 2, 0, -2, 3, 5, 1, 0, -2, 2], [-4, 0, 10, 6, 0, 4, -2, -4, 4, -4, 2, -2], [6, -1, -3, 0, 1, -4, -8, 1, -1, 0, 1, 1], [-7, -2, 6, 2, 6, -1, 0, -4, -7, -2, -4, -4], [6, 1, -3, 2, 3, -4, -8, 3, -1, -2, 3, 3], [-4, 0, 2, -4, 0, -4, -4, 7, 4, 2, 0, 0], [-11, -1, 1, -2, -3, 1, -4, 1, 0, 2, 1, 1], [-3, 0, -2, -2, 0, 0, -5, 1, 3, 3, -4, 4], [-13, -2, -2, 1, 4, -3, -7, 1, -1, -1, 1, 1], [-4, 0, 6, 3, 0, 1, -2, -4, 4, 0, -3, 3], [-7, -3, 3, 2, 7, -3, -4, -3, -6, -2, -3, -3], [-6, 0, 1, -1, 0, 0, -7, 0, 6, 4, -2, 2], [-2, 2, 4, 0, -2, 6, 10, 0, 2, 0, 0, 0], [3, 0, -5, -2, 0, 1, 0, -6, -3, -1, -2, 2], [19, 5, 13, 2, -1, 5, 6, -1, -8, -2, -1, -1], [6, 1, -1, -1, -3, 4, 7, 1, 5, 1, 1, 1], [-5, 1, -3, -1, -3, -5, -3, -2, -2, 1, -2, -2], [6, 4, 4, 0, -4, 2, 4, 0, -2, 0, 0, 0], [12, 4, 8, 2, 0, 2, 6, -2, -6, -2, -2, -2], [-1, 0, 2, 4, 0, 3, 0, -8, 1, 0, 4, -4], [20, -2, -2, 0, 2, 0, 0, 0, 2, 0, 0, 0], [4, 0, 1, 4, 0, -3, 11, -5, -4, 2, 4, -4], [0, 1, -7, -2, -5, 0, -4, 5, 7, 2, 5, 5], [2, 0, -1, 0, 0, -3, 5, -3, -2, 2, 0, 0], [3, 0, 2, 4, 0, 3, 4, -4, -3, -4, 0, 0], [7, 0, -2, -1, -2, 1, 5, -1, 3, 1, -1, -1], [4, 4, 6, 4, 4, 6, 14, 0, 0, -4, 0, 0], [-4, 0, -10, -4, 0, 6, -14, 8, 4, 4, 0, 0], [-11, 2, 2, 2, 2, -1, -4, 2, -3, -2, 2, 2], [1, 0, -8, -2, 0, 1, -2, 4, -1, 4, -2, 2], [12, 0, 6, 2, 4, 2, 8, -4, -4, -2, -4, -4], [-1, 0, -3, 2, 0, 2, -1, 1, 1, 4, 0, 0], [5, 0, 3, -2, 0, -3, 6, -14, -5, -7, 0, 0], [2, 0, 6, -1, 0, -7, 8, -3, -2, -2, 0, 0], [1, -5, 5, -2, 1, 1, 0, -5, -4, 2, -5, -5], [-2, 0, 6, 6, 0, 2, 2, -14, 2, 0, 0, 0], [0, 0, -1, 3, 0, 6, -3, 5, 0, -2, 1, -1], [15, -2, 6, -1, 0, 5, 5, -2, -1, 1, -2, -2], [-1, 0, 5, 2, 0, -1, 2, 2, 1, -1, -2, 2], [-4, 0, -2, -7, 0, -3, -8, -1, 4, 2, 0, 0], [-1, -3, 0, 0, 3, 6, 7, 1, 6, 0, 1, 1], [8, -4, 4, 2, 8, -4, 6, -10, -8, -2, -10, -10], [-3, 0, 1, 0, 0, 0, -3, 7, 3, 2, -2, 2], [-5, -6, -6, -8, -10, 1, -6, 0, 7, 8, 0, 0], [-11, -1, -8, -4, -7, -6, -15, 3, 2, 4, 3, 3], [-4, 0, 7, 2, 0, -3, 1, 8, 4, 2, -4, 4], [1, -4, 12, -1, 2, 5, 5, -6, -7, 1, -6, -6], [-1, 0, 1, 4, 0, 7, -4, 4, 1, -3, 2, -2], [10, 0, -2, 3, 0, 3, 10, -6, -10, -8, 3, -3], [1, 0, 4, -6, 0, -8, 3, 13, -1, -3, -2, 2], [3, 0, 9, 6, 0, -1, 10, -8, -3, -5, 0, 0], [4, 0, 6, 3, 0, -7, 14, 0, -4, 0, 1, -1], [-10, 10, 0, 4, -2, 0, 6, 4, 0, -4, 4, 4], [2, 0, -3, 6, 0, 7, 1, -3, -2, 0, -2, 2], [-11, 0, -2, 1, 2, -1, -9, 5, 1, -1, 5, 5], [11, 4, 0, 1, -2, 3, 7, 2, 3, -1, 2, 2], [7, -1, -4, 0, 1, 0, 5, -1, 4, 0, -1, -1], [0, 0, -7, -1, 0, -4, 3, 3, 0, 10, 1, -1], [8, -2, -4, -2, -2, 2, 4, 0, 6, 2, 0, 0], [-4, 0, -4, -2, 0, 6, -12, -8, 4, 0, -2, 2], [0, 0, -11, -5, 0, 0, -5, 10, 0, 6, 2, -2], [3, 1, -7, 1, 1, -1, 3, 2, 6, -1, 2, 2], [-7, 2, -4, 0, -2, -3, -4, 2, 1, 0, 2, 2], [3, 0, 2, 4, 0, 0, 7, -5, -3, -1, 4, -4], [3, -7, -7, -3, 1, -7, -7, -5, 0, 3, -5, -5], [-4, 0, 0, 4, 0, 4, -4, 12, 4, 4, 4, -4], [-5, 1, -3, 2, 3, -3, 2, -1, 0, -2, -1, -1], [1, 0, -4, 4, 0, 2, 3, -5, -1, 5, 2, -2], [-3, 5, -3, -1, -7, 5, 5, 6, 8, 1, 6, 6], [0, 0, 0, 4, 0, 2, 2, -9, 0, 2, 0, 0], [-10, -10, -6, -6, -2, -4, -10, -4, 2, 6, -4, -4], [-6, 0, 1, -7, 0, -6, -7, 6, 6, 4, -2, 2], [2, 0, 2, 5, 0, 5, 2, 5, -2, -4, 0, 0], [-2, 0, 9, 5, 0, 0, 3, 7, 2, -2, -1, 1], [2, 0, -4, -8, 0, -2, -4, -1, -2, -4, -2, 2], [-6, 0, 0, 2, 0, 4, -8, -6, 6, 4, 0, 0], [0, 0, -3, 6, 0, 11, -5, -5, 0, -2, 2, -2], [10, 0, 5, 2, 0, -5, 17, -6, -10, -8, 0, 0], [9, 2, -2, -1, -4, -9, -11, -2, -7, 1, -2, -2], [13, 5, -1, -2, -9, 5, 12, 1, 6, 2, 1, 1], [-7, 3, -5, 2, 1, 3, 6, 5, 8, -2, 5, 5], [-6, 6, 8, 4, 2, 2, 2, 2, -6, -4, 2, 2], [-4, 0, -8, -4, 0, -2, -6, 5, 4, 10, 2, -2], [4, 0, -8, 0, 0, 4, 0, -22, -4, 0, -2, 2], [-10, 2, -8, -4, -10, 0, -10, 8, 8, 4, 8, 8], [6, 0, -14, -2, -4, -2, -4, 6, 12, 2, 6, 6], [3, 0, -1, 2, 0, -1, 6, 4, -3, 1, -2, 2], [-4, 0, 10, -2, 0, -8, 2, 14, 4, 0, 0, 0], [-22, 2, -7, 0, -2, -3, -1, 2, 4, 0, 2, 2], [-2, 0, 2, -9, 0, -9, -2, 5, 2, 0, -2, 2], [0, 8, 12, 0, -8, 4, 4, 0, -8, 0, 0, 0], [7, 0, 8, 4, 0, -2, 13, -15, -7, -9, -2, 2], [8, 0, -1, 8, 0, 5, 11, -11, -8, -4, 6, -6], [8, 0, 8, -2, -4, 10, 14, -2, 2, 2, -2, -2], [4, 0, -5, -5, 0, 0, -1, 3, -4, -4, 1, -1], [9, 0, -12, 2, 4, -13, -8, -2, -1, -2, -2, -2], [3, 0, 3, 4, 0, -3, 10, 12, -3, 1, 4, -4], [-4, 0, 1, 6, 0, 9, -7, -2, 4, 0, 4, -4], [19, 0, -6, -3, -6, 1, -1, 3, 7, 3, 3, 3], [7, 0, 1, 4, 0, -1, 12, 0, -7, -3, -2, 2], [2, -12, -2, -4, 4, -4, -10, -6, -2, 4, -6, -6], [0, 0, 1, -2, 0, 1, -3, 6, 0, -4, -4, 4], [24, 1, -3, 6, 11, -6, 0, -1, -3, -6, -1, -1]]
